import pnmui_bundle from "./panel-material-ui.bundle.js";
const React = pnmui_bundle.React;

export default React;
export const {
  useState,
  useEffect,
  useMemo,
  useCallback,
  useContext,
  useRef,
  useReducer,
  createContext,
  createElement,
  createRef,
  forwardRef,
  cloneElement,
  isValidElement,
  Children,
  Component,
  Fragment,
  PureComponent,
  StrictMode,
  Suspense,
  lazy,
  memo
} = React;
