:root[data-theme="light"],
:root[data-theme="dark"] {
  --holoviz-main-color: #0072b5;
  --pst-color-inline-code: var(--holoviz-main-color);
  --pst-color-link-hover: var(--holoviz-main-color);
  --pst-color-link: var(--holoviz-main-color);
  --pst-color-primary: var(--holoviz-main-color);
  --pst-color-secondary-bg: #d9ebf0; /* Announcement banner */
  --pst-color-secondary-highlight: var(--holoviz-main-color);
  --pst-color-secondary: var(--holoviz-main-color);
  --pst-violet-600: var(--holoviz-main-color); /* back to top hover */
  --sd-color-card-border-hover: var(--holoviz-main-color);
}

.bd-header-announcement {
  color: black;
}

/* Fix breadcrumb overlap https://github.com/holoviz/panel/issues/5199 */
.header-article__inner .header-article-item {
  height: unset;
}

/* Homepage Showcase */

.showcase-table {
  border-spacing: 15px;
}

.showcase-table td {
  border: 0px;
  vertical-align: top;
}

/* Homepage sponsors */

.sd-card-body.sponsor-logo {
  flex-grow: unset;
  margin: auto;
}

.sponsor-logo img {
  min-height: 80px;
  object-fit: contain;
}

/* Tweaks for code cell rendering */

.highlight {
  border-radius: 4px;
}

html[data-theme="light"] .highlight {
  background-color: #263238;
  color: #f8f8f2;
}

pre[id^="codecell"] {
  background-color: unset;
  border: none;
  border-radius: 0.5em;
  color: #f8f8f2;
  box-shadow: none;
  padding: 1.5em;
}

div.highlight button.copybtn.o-tooltip--left:after {
  background-color: #263235;
  border: 1px solid gray;
  border-radius: 4px;
  padding: 5px;
}

button.copybtn {
  background-color: #263238;
}

button.copybtn:hover {
  background-color: #263238;
  color: #f8f8f2;
}

.highlight button.copybtn:hover {
  background: none;
}

.o-tooltip--left:after {
  background: none;
}

video {
  max-width: 100%;
}

#panel-material-ui .output.text_html {
  overflow: hidden;
}

.cell.tag_hide-input.docutils.container > .cell_output > .text_html {
  padding: 0px;
}
